#include <stdlib.h>
#include "paul.h"

int        ANIMATION_SPEED = SINGLE;
int        SHOW_SLOW = 1;
static int TIMEOUT_TAG = 0;
static int ANIFLAG = 0;

int ApplyPicture(GdkWindow *win, GdkImlibImage *im)
{
  GdkPixmap   *pp;
  int          x, y;
  register int w, h;

  g_return_val_if_fail ( win , -1 );

  w = im->rgb_width;
  h = im->rgb_height;
  if ( SHOW_SLOW ) {
    gdk_window_get_size(win, &x, &y);
    if ( x != w || y != h ) gdk_window_resize(win, w, h);
  }
  g_return_val_if_fail ( gdk_imlib_render(im, w, h), -1 );
  pp  = gdk_imlib_move_image(im);
  gdk_window_set_back_pixmap(win, pp, 0);

  gdk_window_clear(win); 
  gdk_imlib_free_pixmap(pp);
  if ( SHOW_SLOW ) gdk_window_set_title(win, im->filename);
  gdk_window_show(win);
  gdk_flush();
/* printf("%s(%i) ApplyPicture(%s)\n", __FILE__, __LINE__, im->filename);*/
  return 0;
}

static int Animation(PAUL *p)
{
  if ( ANIFLAG ) return 1;  /* avoid displaying if timout is faster than display */
   
  ANIFLAG  = 1;
  ApplyPicture((p->show)->window, BILD(p->activ));
  p->activ = (p->activ)->next;
  ANIFLAG  = 0;

  return 1;
}

static int InstallTimeout(PAUL *p)
{
  register long       wait;

  g_return_val_if_fail ( IS_PAUL(p), -1 );
  
  if ( p->sek < 0.01 ) wait = 0; /* about 0.01 seconds needs calculation */
  else                 wait = 1000*(p->sek) - 10; 

  /* add some integer to wait to enable keyboard input in image window */
  TIMEOUT_TAG = gtk_timeout_add(wait, (GtkFunction)Animation, p);
   
  if ( wait > FAST_TIME ) ANIMATION_SPEED = SLOW_ANIMATION;
  else {
    ANIMATION_SPEED = FAST_ANIMATION;
    SHOW_SLOW       = 0;
    gdk_window_set_title((p->show)->window, "Paul animation");
  }
   
  gtk_widget_set_sensitive(p->filelist, FALSE);
  return TIMEOUT_TAG;
}

static void AnimationTimeChanged(GtkWidget *widget, PAUL *p)
{
  if ( TIMEOUT_TAG ) {
    gtk_timeout_remove(TIMEOUT_TAG);
    InstallTimeout(p);
  }
}

int PictureAnimate(GtkWidget *button, PAUL *p)
{
  GtkWidget *b_stop;
   
  g_return_val_if_fail ( IS_PAUL(p), -1 );
  g_return_val_if_fail ( button, -1 );
  g_return_val_if_fail ( GTK_IS_BUTTON(button), -1 );
  g_return_val_if_fail ( (b_stop = gtk_object_get_user_data(GTK_OBJECT(button))), -1 );
  g_return_val_if_fail ( GTK_IS_BUTTON(b_stop), -1 );
  g_return_val_if_fail ( p->filelist, -1 );
  g_return_val_if_fail ( GTK_IS_CLIST(p->filelist), -1 );
  
  if ( p->sek < FAST_TIME && !p->same_size ) {
    g_warning("Please make sure that all images have the same size.\nThis is neccessary for animation speed under %gs.", (double)FAST_TIME);
    gtk_widget_set_sensitive(b_stop, FALSE);
    gtk_widget_set_sensitive(button, TRUE);
    return -1;
  }

  InstallTimeout(p);

  gtk_widget_set_sensitive(p->filelist, FALSE);
  gtk_widget_set_sensitive(b_stop, TRUE);
  gtk_widget_set_sensitive(button, FALSE);
  return 0;
}

int StopPictureAnimate(GtkWidget *button, PAUL *p)
{
  GtkWidget *b_ani;
   
  g_return_val_if_fail ( IS_PAUL(p), -1 );
  g_return_val_if_fail ( button, -1 );
  g_return_val_if_fail ( GTK_IS_BUTTON(button), -1 );
  g_return_val_if_fail ( (b_ani = gtk_object_get_user_data(GTK_OBJECT(button))), -1 );
  g_return_val_if_fail ( GTK_IS_BUTTON(b_ani), -1 );
  g_return_val_if_fail ( p->filelist, -1 );
  g_return_val_if_fail ( GTK_IS_CLIST(p->filelist), -1 );
  
  gtk_timeout_remove(TIMEOUT_TAG);
  TIMEOUT_TAG = 0;
   
  if ( ANIMATION_SPEED == FAST_ANIMATION ) {
    gdk_window_set_title((p->show)->window, (BILD(p->activ))->filename);
    gdk_window_show((p->show)->window);
    SHOW_SLOW = 1;
  }
  ANIMATION_SPEED = SINGLE;
  gtk_widget_set_sensitive(p->filelist, TRUE);

  gtk_widget_set_sensitive(p->filelist, TRUE);
  gtk_widget_set_sensitive(b_ani, TRUE);
  gtk_widget_set_sensitive(button, FALSE);
  
  return 0;
}


void _ParameterChanged(GtkButton *button, GtkWidget *spinner)
{
  PAUL               *p;

  g_return_if_fail ( button ) ;
  g_return_if_fail ( GTK_IS_BUTTON(button) );
  g_return_if_fail ( (p = gtk_object_get_user_data(GTK_OBJECT(button))) );
  g_return_if_fail ( IS_PAUL(p) );
  g_return_if_fail ( spinner );
  g_return_if_fail ( GTK_IS_SPIN_BUTTON(spinner) );

  p->sek = (double)gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(spinner));
}

void AnimationTimeParameterCallback(PAUL *p)
{
  static GtkWidget *window = NULL;
  GtkWidget        *button;
   
  GtkWidget     *box, *label, *spinner;
  GtkAdjustment *adj;
  
  if ( window ) {
    gtk_widget_destroy(window);
    window = NULL;
  }

  if ( p->sek <= 0 ) p->sek = 0.0;
   
  window = gtk_dialog_new();
  gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_MOUSE);
  gtk_signal_connect(GTK_OBJECT(window), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroyed),
                     &window);

  gtk_container_border_width(GTK_CONTAINER(GTK_DIALOG(window)->vbox), 5);


  box = gtk_hbox_new(FALSE, 0);
  gtk_container_border_width(GTK_CONTAINER(box), 5);
  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(window)->vbox), box);

  label   = gtk_label_new("Delay time [s]:");
  gtk_box_pack_start(GTK_BOX(box), label, TRUE, TRUE, 5);

  adj = (GtkAdjustment *)gtk_adjustment_new(p->sek, 0.0, 10000.0, 0.1, 0.0, 0.0);
  spinner   = gtk_spin_button_new(adj, 0.1, 2);
  gtk_widget_set_usize(spinner, 100, 0);
  gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(spinner), GTK_UPDATE_ALWAYS);
  gtk_box_pack_start(GTK_BOX(box), spinner, TRUE, TRUE, 5);

  box = gtk_hbox_new (FALSE, 0);
  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(window)->action_area), box);
  
  button = gtk_button_new_with_label("OK");
  gtk_object_set_user_data(GTK_OBJECT(button), p);
  gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(_ParameterChanged), spinner);
  gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(AnimationTimeChanged), p);
  gtk_signal_connect_object(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy),
                            GTK_OBJECT(window));
  gtk_box_pack_start(GTK_BOX(box), button, TRUE, TRUE, 5);
  GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
  gtk_widget_grab_default(button);

  button = gtk_button_new_with_label("Cancel");
  gtk_signal_connect_object_after(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy),
                                  GTK_OBJECT(window));
  gtk_box_pack_start(GTK_BOX(box), button, TRUE, TRUE, 5);
  GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);

  gtk_widget_show_all(window);
}

