#include <gdk_imlib.h>
#include <getopt.h>

#include "paul.h"

static void ConfigureCallback(GtkWidget *show, GdkEventConfigure *event, PAUL *p)
{
  static         int w, h;
  GdkImlibImage *bild;

  g_return_if_fail ( GTK_IS_WINDOW(show) );
  g_return_if_fail ( IS_PAUL(p) );
  g_return_if_fail ( event && event->type == GDK_CONFIGURE ) ;
  g_return_if_fail ( (bild = BILD(p->activ)) );

  if ( event->width != bild->W || event->height != bild->H ) {
    w = event->width;
    h = event->height;
    gdk_imlib_apply_image(bild, p->show->window);
  }
}

static void FileListSelectChild(GtkWidget *list, gint row, gint col, GdkEventButton *event, PAUL *p)
{
  g_return_if_fail ( list );
  g_return_if_fail ( GTK_IS_CLIST(list) );
  g_return_if_fail ( IS_PAUL(p) );
  g_return_if_fail ( col <= 0 );
  g_return_if_fail ( p->show ) ;
  g_return_if_fail ( GTK_IS_WINDOW(p->show) ) ;
  

  if ( !GTK_CLIST(p->filelist)->rows ) return ;
  p->activ  = (GList *)gtk_clist_get_row_data(GTK_CLIST(p->filelist), row);
  ApplyPicture(p->show->window, BILD(p->activ));
}

static GList *ReadPicFiles(int nfiles, char **efiles, double sek)
{
  GdkImlibImage *im;
  char         **fp, **fip;
  GList         *piclist = NULL;

  for ( fip = (fp = efiles) + nfiles; fp < fip; fp++ ) {
    if ( !(im = gdk_imlib_load_image(*fp)) ) {
      g_warning("Unable to load %s.", *fp);
      continue;
    }
    piclist = g_list_append(piclist, im);
  }
  return piclist;
}

GdkImlibImage *_BILD(GList *l, char *file, int line) 
{
  if ( !l ) return NULL;
  if ( (l->data) ) return (GdkImlibImage *)(l->data);
  g_warning("No GdkImlibImage in list (file %s, line %i)!", file, line);
  return NULL;
}

int main(int argc, char **argv)
{
  PAUL                 paul;
  int                  i, x = -1, y = -1;
  GList               *pl = NULL;
  GtkWidget           *PaulWindow, *vbox, *b_ani, *b_stop, *hbox;
  GtkItemFactory      *item_factory;
  GtkAccelGroup       *accel_group;
  GtkItemFactoryEntry  menu_items[] = {
    {"/_Quit", "<control>Q", gtk_exit,   0},
    {"/_Animation time", NULL,  AnimationTimeParameterCallback, 0},
  };

  paul.id        = PAUL_ID;
  paul.sek       = -1.0;               /* animation delay time between two images */
  paul.same_size = 1;

  while ( (i = getopt(argc, argv, "w::")) != EOF )
    switch (i) {
      case 'w':
        if ( !optarg ) { paul.sek = 0; break; }
        if ( sscanf(optarg, "%lg", &(paul.sek) ) != 1) return -1; 
        break;
      case '?':
      default:
        g_warning("%s", optarg);
        return -1;
    }
    /* Ende while(getopt()) */

  gtk_init(&argc, &argv);
  gdk_imlib_init();

  paul.piclist = ReadPicFiles(argc - optind, argv + optind, paul.sek);
 
  if ( !(paul.piclist) || !((paul.piclist)->next) ) {
    g_warning("Please input two images at command line."); 
    exit(-1);
  }

  gtk_widget_push_visual(gdk_imlib_get_visual());
  gtk_widget_push_colormap(gdk_imlib_get_colormap());

  PaulWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);

  gtk_signal_connect(GTK_OBJECT(PaulWindow), "destroy", GTK_SIGNAL_FUNC(gtk_exit), &paul);
  gtk_signal_connect(GTK_OBJECT(PaulWindow), "delete-event", GTK_SIGNAL_FUNC(gtk_true), NULL);

  accel_group = gtk_accel_group_new();
  item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>", accel_group);
  gtk_item_factory_create_items(item_factory, sizeof(menu_items)/sizeof(menu_items[0]), menu_items, &paul);
  gtk_accel_group_attach (accel_group, GTK_OBJECT(PaulWindow));

  vbox = gtk_vbox_new (FALSE, 1);
  gtk_box_pack_start(GTK_BOX(vbox), gtk_item_factory_get_widget(item_factory, "<main>"),
			  FALSE, FALSE, 0);
  gtk_container_add(GTK_CONTAINER(PaulWindow), vbox);
  gtk_container_border_width(GTK_CONTAINER(vbox), 3);

  paul.filelist = (GtkWidget *)gtk_clist_new(1);
  gtk_box_pack_start(GTK_BOX(vbox), paul.filelist, FALSE, FALSE, 0);
  gtk_object_set_user_data(GTK_OBJECT(paul.filelist), &paul);

  for ( pl = paul.piclist; pl; pl = pl->next ) {
    if ( (i = gtk_clist_append(GTK_CLIST(paul.filelist), &(BILD(pl)->filename))) < 0 ) exit(-1);
    gtk_clist_set_row_data(GTK_CLIST(paul.filelist), i, pl);
  }
  gtk_signal_connect(GTK_OBJECT(paul.filelist), "select_row", (GtkSignalFunc) FileListSelectChild, &paul); 

  paul.show = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_usize(paul.show, BILD(paul.piclist)->W, BILD(paul.piclist)->H);
  
  gtk_widget_set_events(paul.show, gtk_widget_get_events(paul.show) | GDK_BUTTON_MOTION_MASK |
			 GDK_POINTER_MOTION_HINT_MASK | GDK_POINTER_MOTION_MASK |
			 GDK_BUTTON_PRESS_MASK );

  paul.activ = paul.piclist;
  gtk_widget_show(paul.show);
  gtk_clist_select_row(GTK_CLIST(paul.filelist), 0, 0);

  gdk_window_set_hints((paul.show)->window, 0, 0, 0, 0, 0, 0, GDK_HINT_MIN_SIZE);
  gtk_widget_set_app_paintable(paul.show, TRUE);   
  gtk_signal_connect(GTK_OBJECT(paul.show), "configure_event", GTK_SIGNAL_FUNC(ConfigureCallback), &paul);

  pl = paul.piclist;
  while ( pl ) {
    register  GdkImlibImage *bild = BILD(pl);

    if ( x == -1 ) {
      x = bild->W;
      y = bild->H;
    } else {
      if ( x != bild->W || y != bild->H ) {
        paul.same_size = 0;
        break;
      }
    }
    pl = pl->next;
  }
  (pl = g_list_last(paul.piclist))->next = paul.piclist; /* ringlist !! */
  (paul.piclist)->prev = pl;

  hbox = gtk_hbox_new(TRUE, 10);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

  b_ani = gtk_button_new_with_label("Animate");
  gtk_box_pack_start(GTK_BOX(hbox), b_ani, FALSE, FALSE, 0);
  gtk_signal_connect(GTK_OBJECT(b_ani), "clicked", GTK_SIGNAL_FUNC(PictureAnimate), &paul);  

  b_stop = gtk_button_new_with_label("Stop");
  gtk_box_pack_start(GTK_BOX(hbox), b_stop, FALSE, FALSE, 0);
  gtk_signal_connect(GTK_OBJECT(b_stop), "clicked", GTK_SIGNAL_FUNC(StopPictureAnimate), &paul);  
  gtk_object_set_user_data(GTK_OBJECT(b_ani), b_stop);
  gtk_object_set_user_data(GTK_OBJECT(b_stop), b_ani);

  if ( IsAnimation(paul.sek) ) PictureAnimate(b_ani, &paul) ;
  else                         gtk_widget_set_sensitive(b_stop, FALSE);

  gtk_widget_show_all(PaulWindow);

  gtk_main();
  return 0;
}




